/* bin2hex convertion utility, v0.1 2006/11/13 */
/* stellae (Noel LEMOUEL) / released under GPL */

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#define RECORD_LENGTH 0x10

/* ------- forwards ------- */
static void usage ( char* );

int main(int argc , char ** argv)
{
   int i;

   FILE* f_stdin;
   struct stat binFileInfo;

   unsigned char bin_record[RECORD_LENGTH];
   unsigned int bin_record_len;

   unsigned int records_remainder;

   unsigned int record_address = 0;
   unsigned int sum;

   if (argc != 2)
   {
      usage(argv[0]);
      return -1;
   }

   if (stat(argv[1], &binFileInfo) == -1)
   {
      if (errno == ENOENT)
      {
         (void) fprintf( stderr, "error: '%s' does not exist\r\n", argv[1] );
      }else{
         (void) fprintf( stderr, "error: stat() fails for '%s'\r\n", argv[1] );
      }
      return -1;
   }

   /* checking S_ISLNK(m) is not required since 'stat' follows sym. links */
   if (!S_ISREG(binFileInfo.st_mode))
   {
      (void) fprintf( stderr, "error: '%s' is a not a regular file\r\n", argv[1]);
      return -1;
   }

   if ((f_stdin = fopen(argv[1], "r")) == NULL)
   {
      (void) fprintf( stderr, "error: '%s' open failed\r\n", argv[1] );
      return -1;
   }

   records_remainder = binFileInfo.st_size%RECORD_LENGTH;

   while (fread(bin_record, RECORD_LENGTH, 1, f_stdin))
   {
      printf(":%02X", RECORD_LENGTH);  /* Start code + Byte count */
      printf("%04X", record_address);  /* Address */
      printf("00");                    /* Type */
      sum = RECORD_LENGTH + ((record_address >> 8) & 0xff) + (record_address & 0xff);
      for(i=0; i<RECORD_LENGTH; i++)
      {
         sum+= bin_record[i];         /* sum+= ~(bytes_array[i])+1; */      
         printf("%02X", bin_record[i]);
      }
      printf("%02X\r\n", (-sum & 0xff));
      record_address += RECORD_LENGTH;
   }
   if(records_remainder)
   {
      printf(":%02X", records_remainder); /* Start code + Byte count */
      printf("%04X", record_address);     /* Address */
      printf("00");                       /* Type */
      sum = records_remainder + ((record_address >> 8) & 0xff) + (record_address & 0xff);
      for(i=0; i<records_remainder; i++)
      {
         sum+= bin_record[i];         /* sum+= ~(bytes_array[i])+1; */      
         printf("%02X", bin_record[i]);
      }
      printf("%02X\r\n", (-sum & 0xff));
   }
   printf(":00000001FF\r\n");

   fclose(f_stdin);
   return 0;
}

static void
usage ( char* argv0 )
{
   (void) fprintf( stderr, "usage: %s [bin_file_name]\r\n", argv0 );
   exit(1);
}

